# Copyright 2022 The alphaclops Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import alphaclops


def test_drop():
    q1 = alphaclops.NamedQubit('q1')
    q2 = alphaclops.NamedQubit('q2')
    alphaclops.testing.assert_same_circuits(
        alphaclops.drop_empty_moments(
            alphaclops.Circuit(
                alphaclops.Moment(), alphaclops.Moment(), alphaclops.Moment([alphaclops.CNOT(q1, q2)]), alphaclops.Moment()
            )
        ),
        alphaclops.Circuit(alphaclops.Moment([alphaclops.CNOT(q1, q2)])),
    )


def test_drop_empty_moments():
    q1, q2 = alphaclops.LineQubit.range(2)
    c_nested = alphaclops.FrozenCircuit(
        alphaclops.Moment(), alphaclops.Moment(), alphaclops.Moment([alphaclops.CNOT(q1, q2)]), alphaclops.Moment()
    )
    c_nested_dropped = alphaclops.FrozenCircuit(alphaclops.CNOT(q1, q2))
    c_orig = alphaclops.Circuit(
        c_nested,
        alphaclops.CircuitOperation(c_nested).repeat(6).with_tags("nocompile"),
        c_nested,
        alphaclops.CircuitOperation(c_nested).repeat(5).with_tags("preserve_tag"),
        c_nested,
    )
    c_expected = alphaclops.Circuit(
        c_nested_dropped,
        alphaclops.CircuitOperation(c_nested).repeat(6).with_tags("nocompile"),
        c_nested_dropped,
        alphaclops.CircuitOperation(c_nested_dropped).repeat(5).with_tags("preserve_tag"),
        c_nested_dropped,
    )
    context = alphaclops.TransformerContext(tags_to_ignore=("nocompile",), deep=True)
    alphaclops.testing.assert_same_circuits(alphaclops.drop_empty_moments(c_orig, context=context), c_expected)
